<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://www.dicha.gr/
 * @since      1.0.0
 *
 * @package    Dc_Paybybank
 * @subpackage Dc_Paybybank/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Dc_Paybybank
 * @subpackage Dc_Paybybank/admin
 * @author     Digital Challenge <info@dicha.gr>
 */
class Dc_Paybybank_Admin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $plugin_name The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $version The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @param string $plugin_name The name of this plugin.
	 * @param string $version The version of this plugin.
	 *
	 * @since    1.0.0
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version     = $version;

	}

	/**
	 * Display field value on the order edit page
	 *
	 * @param WC_Order $order
	 */
	public function dc_paybybank_checkout_field_display_admin_order_meta( $order ) {
		$method = get_post_meta( $order->get_id(), '_payment_method', true );
		if ( $method != 'dc-paybybank' ) {
			return;
		}

		$reference_code = get_post_meta( $order->get_id(), 'dc_reference_code', true );

		echo '<p><strong>' . __( 'PayByBank Reference Code', 'dc-paybybank' ) . ':</strong> ' . $reference_code . '</p>';
	}


	/**
	 * Add PayByBank as a new payment gateway
	 *
	 * @param $methods
	 *
	 * @return array
	 */
	public function add_dc_paybybank_gateway_class( $methods ) {
		$methods[] = 'WC_Gateway_DC_PayByBank';

		return $methods;
	}


	/**
	 *  Register a new route to get the payment success call from PayByBank service
	 */
	public function api_register_paybybank_payment_url() {
		register_rest_route( 'dc-paybybank', '/paybybank-success/', array(
			'methods'  => 'POST',
			'callback' => array( $this, 'get_paybybank_request' )
		) );
	}

	/**
	 * Receive the call from PayByBank service for successful payment and send the appropriate message.
	 *
	 * @param $request_data
	 *
	 * @return string
	 */
	public function get_paybybank_request( $request_data ) {
		$order_id     = $request_data->get_params()['merchantOrderId'];
		$order_status = $request_data->get_params()['omtTransactionBank']['merchantOrderStatus'];
		$order        = new WC_Order( $order_id );
		if ( $order_status == 'PAID' || $order_status == 'COMPLETED' ) {
			$gateway = new WC_Gateway_DC_PayByBank();
			if ( $order->get_status() == 'paybybank-paid' ) {
				$order->add_order_note( __( 'Order paid successfully by PayByBank.', 'dc-paybybank' ) );
			} else {
				$order->update_status( 'wc-paybybank-paid', __( 'Order paid successfully by PayByBank. Status changed automatically after client payment.', 'dc-paybybank' ) );
			}

			return 'OK';
		} else {
			$order->add_order_note( sprintf( __( 'Problem in receiving request from PayByBank. OrderId: %s, OrderStatus: %s', 'dc-paybybank' ), [
				$order_id,
				$order_status
			] ) );

			return 'Error';
		}

	}

	/**
	 * Add a new order meta box action for PayByBank from order details' page
	 *
	 * @param $actions
	 *
	 * @return mixed
	 */
	public function dc_paybybank_add_order_action( $actions ) {
		global $post;
		$order = new WC_Order ( $post->ID );
		if ( $order->get_payment_method() == 'dc-paybybank' && ( $order->get_status() != 'completed' && $order->get_status() != 'paybybank-paid' ) ) {
			$actions['dc_paybybank_get_order_status'] = __( 'Check payment for PayByBank', 'dc-paybybank' );
		}

		return $actions;
	}

	/**
	 * Create a curl request to PayByBank API to get payment status
	 *
	 * @param WC_Order $order
	 */
	public function dc_paybybank_get_order_status( WC_Order $order ) {
		$gateway = new WC_Gateway_DC_PayByBank();
		if ( $gateway->mode == 'yes' ) {
			$base_path = DC_PAYBYBANK_BASE_TEST_PATH;
		} else {
			$base_path = DC_PAYBYBANK_BASE_PATH;
		}
		$result       = wp_remote_get( $base_path . DC_PAYBYBANK_SUBMIT_ORDER_PATH . $gateway->api_key . '/' . $order->get_id(), array( 'sslverify' => false ) );
		$final_result = json_decode( $result['body'], true );
		if ( is_array( $final_result ) && sizeof( $final_result ) == 1 ) {
			if ( $final_result[0]['omtTransactionBank']['merchantOrderStatus'] == 'PAID' || $final_result[0]['omtTransactionBank']['merchantOrderStatus'] == 'COMPLETED' ) {
				$order->update_status( 'wc-paybybank-paid', __( 'Order paid successfully. Status changed automatically upon order request.', 'dc-paybybank' ) );
				add_filter( 'redirect_post_location', array( __CLASS__, 'set_dc_paybybank_success_message' ) );
			} else {
				add_filter( 'redirect_post_location', array( __CLASS__, 'set_dc_paybybank_pending_message' ) );
			}
		} else {
			add_filter( 'redirect_post_location', array( __CLASS__, 'set_dc_paybybank_failure_message' ) );
		}
	}

	/**
	 * @param $location
	 *
	 * @return string
	 */
	public function set_dc_paybybank_success_message( $location ) {
		return add_query_arg( 'message', 12, $location );
	}

	/**
	 * @param $location
	 *
	 * @return string
	 */
	public function set_dc_paybybank_failure_message( $location ) {
		return add_query_arg( 'message', 13, $location );
	}

	/**
	 * @param $location
	 *
	 * @return string
	 */
	public function set_dc_paybybank_pending_message( $location ) {
		return add_query_arg( 'message', 14, $location );
	}

	/**
	 * WooCommerce Add fee to checkout for a gateway ID
	 */
	public function dc_paybybank_add_checkout_fee_for_gateway() {
		$chosen_gateway = WC()->session->get( 'chosen_payment_method' );
		if ( $chosen_gateway == 'dc-paybybank' ) {
			$gateway = new WC_Gateway_DC_PayByBank();
			if ( isset( $gateway->extra_fee ) && $gateway->extra_fee > 0 ) {
				WC()->cart->add_fee( __( 'PayByBank Fee', 'dc-paybybank' ), $gateway->extra_fee );
			}
		}
	}

	/**
	 * The message for failure of order action
	 *
	 * @param $messages
	 *
	 * @return mixed
	 */
	public function dc_paybybank_post_updated_messages( $messages ) {
		$messages['shop_order'][12] = __( 'Order has successfully paid via PayByBank.', 'dc-paybybank' );
		$messages['shop_order'][13] = __( 'Cannot check the status of order in PayByBank due to Service error. Please contact the support.', 'dc-paybybank' );
		$messages['shop_order'][14] = __( 'Order is still Pending and not paid.', 'dc-paybybank' );

		return $messages;
	}

	/**
	 * Display the merchant PaymentURL who needed to PayByBank
	 *
	 * @param $array
	 *
	 * @return mixed
	 */
	function filter_woocommerce_settings_api_form_fields_id( $array ) {

		$merchant_payment_url = ( isset( $_SERVER['HTTPS'] ) && $_SERVER['HTTPS'] === 'on' ? "https" : "http" ) . "://$_SERVER[HTTP_HOST]/wp-json/dc-paybybank/paybybank-success/";
		?>
        <h4><?php echo __( 'Merchant PaymentURL (send this address to PayByBank)', 'dc-paybybank' ) ?></h4>
        <p><?php echo $merchant_payment_url ?></p>
		<?php

		return $array;
	}

	/**
	 * Output for the order received page.
	 */
	public function dc_paybybank_add_thankyou_message( $order_id ) {
		$gateway = new WC_Gateway_DC_PayByBank();
		if ( $gateway->instructions ) {
			echo wpautop( wptexturize( $gateway->instructions ) );
		}

		$reference_code = get_post_meta( $order_id, 'dc_reference_code', true );
		echo wpautop( wptexturize( __( 'To pay for your order and execute it promptly with no additional procedures and no payment costs, you can make payments to your bank via e-banking or phone banking by selecting Payments> PayByBank and entering the payment code below and the exact one amount of payment. <br> We will automatically receive your payment and your order will be forwarded immediately. <br> There are no payment costs, as long as the bank is Greek and you choose to pay via web banking or phone banking or your bank\'s app and not in a physical store. This applies to all Greek banks. <br> <br> Please pay the bank of your choice with the following payment code:', 'dc-paybybank' ) . ' <b>' . $reference_code . '</b>' ) );

	}

	/**
	 * Add content to the WC emails.
	 *
	 * @access public
	 *
	 * @param WC_Order $order
	 * @param bool $sent_to_admin
	 * @param bool $plain_text
	 * @param WC_Email $email
	 */
	public function dc_paybybank_add_email_instructions( $order, $sent_to_admin, $plain_text, $email ) {
		$gateway = new WC_Gateway_DC_PayByBank();
		if ( $gateway->instructions && ! $sent_to_admin && 'dc-paybybank' == $order->get_payment_method() && 'wc-' . $order->get_status() == $gateway->order_status ) {
			echo wpautop( wptexturize( $gateway->instructions ) ) . PHP_EOL;
		}

		$reference_code = get_post_meta( $order->get_id(), 'dc_reference_code', true );
		if ( 'wc-' . $order->get_status() == $gateway->order_status && 'dc-paybybank' == $order->get_payment_method() ) {
			if ( ! $sent_to_admin ) {
				echo wpautop( wptexturize( __( 'To pay for your order and execute it promptly with no additional procedures and no payment costs, you can make payments to your bank via e-banking or phone banking by selecting Payments> PayByBank and entering the payment code below and the exact one amount of payment. <br> We will automatically receive your payment and your order will be forwarded immediately. <br> There are no payment costs, as long as the bank is Greek and you choose to pay via web banking or phone banking or your bank\'s app and not in a physical store. This applies to all Greek banks. <br> <br> Please pay the bank of your choice with the following payment code:', 'dc-paybybank' ) . ' <b>' . $reference_code . '</b>' ) );
			} else {
				echo wpautop( wptexturize( __( 'PayByBank Reference Code', 'dc-paybybank' ) . ' : ' . $reference_code ) );
			}
		} else if ( $order->get_status() == 'paybybank-paid' && 'dc-paybybank' == $order->get_payment_method() ) {
			echo wpautop( wptexturize( __( 'Order paid successfully by PayByBank.', 'dc-paybybank' ) ) );
		}
	}

	/**
	 * Register PayByBank paid status
	 */
	public function dc_paybybank_register_paybybank_paid_order_status() {
		register_post_status( 'wc-paybybank-paid', array(
			'label'                     => __( 'Paid by PayByBank', 'dc-paybybank' ),
			'public'                    => true,
			'show_in_admin_status_list' => true,
			'show_in_admin_all_list'    => true,
			'exclude_from_search'       => false,
		) );
	}

	/**
     * Add new paid status in WooCommerce orders
     *
	 * @param $order_statuses
	 *
	 * @return array
	 */
	public function dc_paybybank_add_paybybank_paid_to_order_statuses( $order_statuses ) {

		$new_order_statuses = array();

		foreach ( $order_statuses as $key => $status ) {

			$new_order_statuses[ $key ] = $status;

			if ( 'wc-processing' === $key ) {
				$new_order_statuses['wc-paybybank-paid'] = __( 'Paid by PayByBank', 'dc-paybybank' );
			}
		}

		return $new_order_statuses;
	}

	/**
     * Email instructions override
     *
	 * @param $order_id
	 * @param $order
	 */
	public function dc_paybybank_email_order_status_paybybank_paid( $order_id, $order ) {

		$heading = __( 'Paid by PayByBank', 'dc-paybybank' );
		$subject = __( 'Paid by PayByBank', 'dc-paybybank' );

		// Get WooCommerce email objects
		$mailer = WC()->mailer()->get_emails();

		// Use one of the active emails e.g. "Customer_Completed_Order"
		// Wont work if you choose an object that is not active
		// Assign heading & subject to chosen object
		$mailer['WC_Email_Customer_Processing_Order']->heading             = $heading;
		$mailer['WC_Email_Customer_Processing_Order']->settings['heading'] = $heading;
		$mailer['WC_Email_Customer_Processing_Order']->subject             = $subject;
		$mailer['WC_Email_Customer_Processing_Order']->settings['subject'] = $subject;
		$mailer['WC_Email_Customer_Processing_Order']->trigger( $order_id );

		$mailer['WC_Email_New_Order']->heading             = $heading;
		$mailer['WC_Email_New_Order']->settings['heading'] = $heading;
		$mailer['WC_Email_New_Order']->subject             = $subject;
		$mailer['WC_Email_New_Order']->settings['subject'] = $subject;
		$mailer['WC_Email_New_Order']->trigger( $order_id );
	}

}
